unit Sample4;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blocks, Statsim, ComCtrls;

type
  TForm1 = class(TForm)
    Memo: TMemo;
    Button1: TButton;
    MultiRand: TMultiRand;
    Label1: TLabel;
    ProgressBar: TProgressBar;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    EditSim: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    Entrance: TGenerator;
    Hall: TQueue;
    Barber: TServer;
    ExitDoor: TTerminator;
    Model: TModel;
    procedure Button1Click(Sender: TObject);
    procedure EditSimExit(Sender: TObject);
    procedure EntranceBeforeTimeGoOn(Sender: TAggregate);
    procedure EntranceExit(Sender: TBlock; Trans: TTransaction);
    procedure EntranceRouting(Sender: TBlock; Trans: TTransaction);
    procedure HallRouting(Sender: TBlock; Trans: TTransaction);
    procedure BarberEnter(Sender: TBlock; Trans: TTransaction);
    procedure BarberRouting(Sender: TBlock; Trans: TTransaction);
    procedure ModelNewTime(Sender: TAggregate; Trans: TTransaction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;



var
  Form1: TForm1;
  ArrivalTime: real;    // Avarage arrival time
  ServiceTime: real;    // Average service time
  LimitTime: real;      // Simulation time


implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
var i: integer;
begin
   try
      LimitTime:=StrToFloat(Trim(EditSim.Text));
   except
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;
   if LimitTime<=0.0 then
   begin
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;

   ArrivalTime:=10.0;
   ServiceTime:=10.0;

   Memo.Lines.Clear;
   for i:=0 to 5 do
   begin
      Model.Simulate(LimitTime);
      Memo.Lines.Add('Average arrival time: '+FormatFloat('0.00',ArrivalTime)+
                     '   Average queue length: '+
                     FormatFloat('0.00',Hall.AverageCount) );
      Model.Reset;
      ArrivalTime:=ArrivalTime+1.0;
   end;
end;

procedure TForm1.EditSimExit(Sender: TObject);
begin
   try
      LimitTime:=StrToFloat(Trim(EditSim.Text));
   except
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;
   if LimitTime<=0.0 then
   begin
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
   end;
end;

procedure TForm1.EntranceBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.EntranceExit(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(ArrivalTime));
end;

procedure TForm1.EntranceRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Hall);
end;

procedure TForm1.HallRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Barber);
end;

procedure TForm1.BarberEnter(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(ServiceTime));
end;

procedure TForm1.BarberRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(ExitDoor);
end;


procedure TForm1.ModelNewTime(Sender: TAggregate; Trans: TTransaction);
var  NewPos: integer;
     Ratio: real;
begin
   Ratio:=ModelTime/LimitTime;
   if Ratio>1.0 then Ratio:=1.0;
   NewPos:=Round(Ratio*100.0);
   with ProgressBar do
      if Position<>NewPos then Position:=NewPos;
end;

end.
