unit Sample8;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blocks, Statsim;

const
  ChangeTime: array[1..5] of real = (0.0,120.0,240.0,420.0,540.0);
  ArrivalMeans: array[1..5] of real = (14.0,12.0,10.0,12.0,14.0);

type
  TForm1 = class(TForm)
    Memo: TMemo;
    Button1: TButton;
    MultiRand: TMultiRand;
    Label1: TLabel;
    Scheduler1: TScheduler;
    Entrance: TGenerator;
    Hall: TQueue;
    Barber: TServer;
    ExitDoor: TTerminator;
    Model: TModel;
    procedure Button1Click(Sender: TObject);
    procedure Scheduler1BeforeTimeGoOn(Sender: TAggregate);
    procedure Scheduler1Planned(Sender: TAggregate);
    procedure EntranceBeforeTimeGoOn(Sender: TAggregate);
    procedure EntranceExit(Sender: TBlock; Trans: TTransaction);
    procedure EntranceRouting(Sender: TBlock; Trans: TTransaction);
    procedure HallRouting(Sender: TBlock; Trans: TTransaction);
    procedure BarberEnter(Sender: TBlock; Trans: TTransaction);
    procedure BarberRouting(Sender: TBlock; Trans: TTransaction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  ArrivalTime: real;
  Counter: integer;

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
begin
   Counter:=1;
   Memo.Lines.Clear;
   Model.Simulate(720.0);
   Memo.Lines.Add('Usage of the barber: '+FormatFloat('0.00',Barber.Usage));
   Memo.Lines.Add('Average queue length: '+FormatFloat('0.00',Hall.AverageCount));
   Memo.Lines.Add('Maximal queue length: '+IntToStr(Hall.MaxCount));
   Memo.Lines.Add('Average waiting time: '+FormatFloat('0.00',Hall.AverageTime));
   Memo.Lines.Add('Deviation of waiting time: '+FormatFloat('0.00',Hall.DeviationTime));
   Memo.Lines.Add('Average waiting time / excluding zero times: '+FormatFloat('0.00',Hall.SAverageTime));
   Memo.Lines.Add('Deviation of waiting time/ excluding zero times: '+FormatFloat('0.00',Hall.SDeviationTime));
   MultiRand.Reset;
   Model.Reset;
end;

procedure TForm1.Scheduler1BeforeTimeGoOn(Sender: TAggregate);
begin
   Sender.NextTime(0.0);  // The first planned event will occur at 0.0
end;

procedure TForm1.Scheduler1Planned(Sender: TAggregate);
begin
   ArrivalTime:=ArrivalMeans[Counter];            // Setting the mean of arrival time
   if Counter<5 then
      Sender.NextTime(ChangeTime[Counter+1]);  // Ordering the next event
   Inc(Counter);
end;

procedure TForm1.EntranceBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.EntranceExit(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(ArrivalTime));
end;

procedure TForm1.EntranceRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Hall);
end;

procedure TForm1.HallRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Barber);
end;


procedure TForm1.BarberEnter(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(10.0));
end;

procedure TForm1.BarberRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(ExitDoor);
end;

end.
